Imports Microsoft.VisualBasic
Imports System
Imports System.Windows.Input

Namespace DemoApp.ViewModel
	''' <summary>
	''' This ViewModelBase subclass requests to be removed 
	''' from the UI when its CloseCommand executes.
	''' This class is abstract.
	''' </summary>
	Public MustInherit Class WorkspaceViewModel
		Inherits ViewModelBase
		#Region "Fields"

		Private _closeCommand As RelayCommand

		#End Region ' Fields

		#Region "Constructor"

		Protected Sub New()
		End Sub

		#End Region ' Constructor

		#Region "CloseCommand"

		''' <summary>
		''' Returns the command that, when invoked, attempts
		''' to remove this workspace from the user interface.
		''' </summary>
		Public ReadOnly Property CloseCommand() As ICommand
			Get
                If _closeCommand Is Nothing Then
                    Dim requestCloseAction As New Action(Of Object)(AddressOf OnRequestClose)
                    _closeCommand = New RelayCommand(requestCloseAction)
                End If

				Return _closeCommand
			End Get
		End Property

		#End Region ' CloseCommand

		#Region "RequestClose [event]"

		''' <summary>
		''' Raised when this workspace should be removed from the UI.
		''' </summary>
		Public Event RequestClose As EventHandler

		Private Sub OnRequestClose()
			Dim handler As EventHandler = Me.RequestCloseEvent
			If handler IsNot Nothing Then
				handler(Me, EventArgs.Empty)
			End If
		End Sub

		#End Region ' RequestClose [event]
	End Class
End Namespace