Imports System.ComponentModel
Imports Microsoft.VisualBasic
Imports System
Imports DemoApp.DataAccess
Imports DemoApp.Model
Imports DemoApp.ViewModel
Imports Microsoft.VisualStudio.TestTools.UnitTesting

Namespace UnitTests
    <TestClass()> _
    Public Class AllCustomersViewModelTests

        Dim notifications As Integer

#Region "Boilerplate Code"

        Public Sub New()
        End Sub

        Private testContextInstance As TestContext

        ''' <summary>
        '''Gets or sets the test context which provides
        '''information about and functionality for the current test run.
        '''</summary>
        Public Property TestContext() As TestContext
            Get
                Return testContextInstance
            End Get
            Set(ByVal value As TestContext)
                testContextInstance = value
            End Set
        End Property

#Region "Additional test attributes"
        '
        ' You can use the following additional attributes as you write your tests:
        '
        ' Use ClassInitialize to run code before running the first test in the class
        ' [ClassInitialize()]
        ' public static void MyClassInitialize(TestContext testContext) { }
        '
        ' Use ClassCleanup to run code after all tests in a class have run
        ' [ClassCleanup()]
        ' public static void MyClassCleanup() { }
        '
        ' Use TestInitialize to run code before running each test 
        ' [TestInitialize()]
        ' public void MyTestInitialize() { }
        '
        ' Use TestCleanup to run code after each test has run
        ' [TestCleanup()]
        ' public void MyTestCleanup() { }
        '
#End Region

#End Region ' Boilerplate Code

        <TestMethod()> _
        Public Sub TestTotalSelectedSales()
            Dim repos As New CustomerRepository(Constants.CUSTOMER_DATA_FILE)
            Dim target As New AllCustomersViewModel(repos)

            notifications = 0
            AddHandler target.PropertyChanged, AddressOf PropertyChangedHandler

            Assert.AreEqual(0.0, target.TotalSelectedSales, "Should be zero when no customers are selected")

            Dim firstCust As CustomerViewModel = target.AllCustomers(0)

            firstCust.IsSelected = True
            Assert.AreEqual(1, notifications, "TotalSelectedSales change notification was not raised")
            Assert.AreEqual(firstCust.TotalSales, target.TotalSelectedSales)

            Dim secondCust As CustomerViewModel = target.AllCustomers(1)
            secondCust.IsSelected = True
            Assert.AreEqual(2, notifications, "TotalSelectedSales change notification was not raised again")
            Assert.AreEqual(firstCust.TotalSales + secondCust.TotalSales, target.TotalSelectedSales)
        End Sub

        <TestMethod()> _
        Public Sub TestNewCustomerIsAdded()
            Dim repos As New CustomerRepository(Constants.CUSTOMER_DATA_FILE)
            Dim target As New AllCustomersViewModel(repos)

            Assert.AreEqual(3, target.AllCustomers.Count, "Test data includes three customers")

            repos.AddCustomer(Customer.CreateCustomer(123.45, "new", "customer", False, "new.customer@email.com"))

            Assert.AreEqual(4, target.AllCustomers.Count, "Adding a customer to the repository increase the Count")
        End Sub

        Private Sub PropertyChangedHandler(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
            If e.PropertyName = "TotalSelectedSales" Then
                notifications += 1
            End If
        End Sub
    End Class
End Namespace