Imports System.Linq
Imports System.Windows.Data
Imports DemoApp.Properties
Imports DemoApp.ViewModel
Imports Microsoft.VisualStudio.TestTools.UnitTesting

Namespace UnitTests
	<TestClass> _
	Public Class MainWindowViewModelTests
		#Region "Boilerplate Code"

		Public Sub New()
		End Sub

		Private testContextInstance As TestContext

		''' <summary>
		'''Gets or sets the test context which provides
		'''information about and functionality for the current test run.
		'''</summary>
		Public Property TestContext() As TestContext
			Get
				Return testContextInstance
			End Get
			Set(ByVal value As TestContext)
				testContextInstance = value
			End Set
		End Property

		#Region "Additional test attributes"
		'
		' You can use the following additional attributes as you write your tests:
		'
		' Use ClassInitialize to run code before running the first test in the class
		' [ClassInitialize()]
		' public static void MyClassInitialize(TestContext testContext) { }
		'
		' Use ClassCleanup to run code after all tests in a class have run
		' [ClassCleanup()]
		' public static void MyClassCleanup() { }
		'
		' Use TestInitialize to run code before running each test 
		' [TestInitialize()]
		' public void MyTestInitialize() { }
		'
		' Use TestCleanup to run code after each test has run
		' [TestCleanup()]
		' public void MyTestCleanup() { }
		'
		#End Region

		#End Region ' Boilerplate Code

		'
		' NOTE: In order to access the auto-generated Strings class in the main assembly, I had to change the
		' Custom Tool associated with Strings.resx from ResXFileCodeGenerator to PublicResXFileCodeGenerator.
		' You can specify the custom tool by selecting that resx file in Solution Explorer, and then view its 
		' properties by opening the Properties Window for that file.
		'

		<TestMethod> _
		Public Sub TestViewAllCustomers()
			Dim target As New MainWindowViewModel(Constants.CUSTOMER_DATA_FILE)
			Dim commandVM As CommandViewModel = target.Commands.First(Function(cvm) cvm.DisplayName = Strings.MainWindowViewModel_Command_ViewAllCustomers)
			commandVM.Command.Execute(Nothing)

			Dim collectionView = CollectionViewSource.GetDefaultView(target.Workspaces)
			Assert.IsTrue(TypeOf collectionView.CurrentItem Is AllCustomersViewModel, "Invalid current item.")
		End Sub

		<TestMethod> _
		Public Sub TestCreateNewCustomer()
			Dim target As New MainWindowViewModel(Constants.CUSTOMER_DATA_FILE)
			Dim commandVM As CommandViewModel = target.Commands.First(Function(cvm) cvm.DisplayName = Strings.MainWindowViewModel_Command_CreateNewCustomer)
			commandVM.Command.Execute(Nothing)

			Dim collectionView = CollectionViewSource.GetDefaultView(target.Workspaces)
			Assert.IsTrue(TypeOf collectionView.CurrentItem Is CustomerViewModel, "Invalid current item.")
		End Sub

		<TestMethod> _
		Public Sub TestCannotViewAllCustomersTwice()
			Dim target As New MainWindowViewModel(Constants.CUSTOMER_DATA_FILE)
			Dim commandVM As CommandViewModel = target.Commands.First(Function(cvm) cvm.DisplayName = Strings.MainWindowViewModel_Command_ViewAllCustomers)
			' Tell the ViewModel to show all customers twice.
			commandVM.Command.Execute(Nothing)
			commandVM.Command.Execute(Nothing)

			Dim collectionView = CollectionViewSource.GetDefaultView(target.Workspaces)
			Assert.IsTrue(TypeOf collectionView.CurrentItem Is AllCustomersViewModel, "Invalid current item.")
            Assert.IsTrue(target.Workspaces.Count = 1, "Invalid number of view models.")
		End Sub

		<TestMethod> _
		Public Sub TestCloseAllCustomersWorkspace()
			' Create the MainWindowViewModel, but not the MainWindow.
			Dim target As New MainWindowViewModel(Constants.CUSTOMER_DATA_FILE)

			Assert.AreEqual(0, target.Workspaces.Count, "Workspaces isn't empty.")

			' Find the command that opens the "All Customers" workspace.
			Dim commandVM As CommandViewModel = target.Commands.First(Function(cvm) cvm.DisplayName = Strings.MainWindowViewModel_Command_ViewAllCustomers)

			' Open the "All Customers" workspace.
			commandVM.Command.Execute(Nothing)
			Assert.AreEqual(1, target.Workspaces.Count, "Did not create viewmodel.")

			' Ensure the correct type of workspace was created.
			Dim allCustomersVM = TryCast(target.Workspaces(0), AllCustomersViewModel)
			Assert.IsNotNull(allCustomersVM, "Wrong viewmodel type created.")

			' Tell the "All Customers" workspace to close.
			allCustomersVM.CloseCommand.Execute(Nothing)
			Assert.AreEqual(0, target.Workspaces.Count, "Did not close viewmodel.")
		End Sub
	End Class
End Namespace